{ **************************************************************************** }
{ *                                                                          * }
{ * TURTLE BRIDGE (Nintendo, Game & Watch Wide Screen) Simulator             * }
{ * by Luca "MADrigal" Antignano, May 2000 (revision: August 2021)           * }
{ *                                                                          * }
{ * http://www.madrigaldesign.it/sim/                                        * }
{ * lucantignano@gmail.com                                                   * }
{ *                                                                          * }
{ * This source code (the "Software") is released as "open source" under the * }
{ * "Creative Common Attribution-NonCommercial-ShareAlike" license 4.0       * }
{ * available at:                                                            * }
{ * https://creativecommons.org/licenses/by-nc-sa/4.0/legalcode              * }
{ *                                                                          * }
{ * In addition to the constraints set by the CC license, you must respect   * }
{ * the following rules:                                                     * }
{ * a. do not offer the Software on a pay-per-play basis.                    * }
{ * b. do not distribute the Software through pre-paid access websites or    * }
{ *    BBS's.                                                                * }
{ * c. do not use the Software in publications of any form unless written    * }
{ *    consent is obtained by the author.                                    * }
{ * d. do not distribute the Software through mirror websites, unless        * }
{ *    written consent is obtained by the author.                            * }
{ * e. do not distribute older versions of the Software through authorized   * }
{ *    mirror websites after newer ones have been made available by the      * }
{ *    author.                                                               * }
{ *                                                                          * }
{ * Excerpt from the CC BY-NC-SA license 4.0                                 * }
{ * ========================================                                 * }
{ * BY: You must give appropriate credit, provide a link to the license, and * }
{ *     indicate if changes were made. You may do so in any reasonable       * }
{ *     manner, but not in any way that suggests the licensor endorses you   * }
{ *     or your use.                                                         * }
{ * NC: You may not use the material for commercial purposes.                * }
{ * SA: If you remix, transform, or build upon the material, you must        * }
{ *     distribute your contributions under the same license as the          * }
{ *     original.                                                            * }
{ *                                                                          * }
{ **************************************************************************** }

unit unit1;
{ this unit contains the main form implementation                              }

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  ExtCtrls, Math, Registry, Jpeg, Pngimage, Fmod, Fmodtypes, StdCtrls, Menus,
  ShellAPI;

type
  TForm1 = class(TForm)
{ * HANDHELD GRAPHICS, INCLUDING BUTTONS                                     * }

    // fullscreen mode items
    btn_game_menu_top:        TImage;
    btn_game_menu_background: TImage;

    im_backdrop:              TImage;
    im_backdrop_logo:         TImage;

    // electronic game
    im_background:            TImage;       im_background_screen_top: TImage;   {1-2}

    btn_game_a_down:          TImage;       btn_game_a_top:           TImage;   {3-4}
    btn_game_b_down:          TImage;       btn_game_b_top:           TImage;   {5-6}
    btn_time_down:            TImage;       btn_time_top:             TImage;   {7-8}
    btn_acl_down:             TImage;       btn_acl_top:              TImage;   {9-10}
                                            btn_alarm_top:            TImage;   {11}

    btn_left_down:            TImage;       btn_left_top:             TImage;   {12-13}
    btn_right_down:           TImage;       btn_right_top:            TImage;   {14-15}

{ * LCD SCREEN GRAPHICS (SPRITES)                                            * }

    // on-screen panel (alarm, time/score, game a/b, etc.)
    im_panel_time_am: TImage; im_panel_time_pm: TImage; im_panel_time_colon: TImage; {16-18}

    im_panel_alarm_1: TImage; im_panel_alarm_2: TImage; im_panel_alarm_3: TImage; im_panel_alarm_4: TImage; im_panel_alarm_5: TImage; {19-23}

    im_panel_digit_1: TImage; im_panel_digit_2: TImage; im_panel_digit_3: TImage; im_panel_digit_4: TImage; {24-27}

    im_panel_miss_0: TImage; im_panel_miss_1: TImage; im_panel_miss_2: TImage; im_panel_miss_3: TImage; {28-31}

    im_panel_game_a: TImage; im_panel_game_b: TImage;                           {32-33}

    im_panel_number_0: TImage; im_panel_number_1: TImage; im_panel_number_2: TImage; im_panel_number_3: TImage; im_panel_number_4: TImage;
    im_panel_number_5: TImage; im_panel_number_6: TImage; im_panel_number_7: TImage; im_panel_number_8: TImage; im_panel_number_9: TImage; {34-43}

    // on-screen items
    im_tourist_left: TImage; im_tourist_right_1: TImage; im_tourist_right_2: TImage; im_tourist_right_3: TImage; {44-47}

    im_hero_0_a: TImage;    im_hero_1_a: TImage;       im_hero_2_a: TImage;       im_hero_3_a: TImage;       im_hero_4_a: TImage;       im_hero_5_a: TImage; {48-53}
    im_hero_0_b: TImage;    im_hero_1_b: TImage;       im_hero_2_b: TImage;       im_hero_3_b: TImage;       im_hero_4_b: TImage;       im_hero_5_b: TImage; {54-59}

    im_baggage_0_a: TImage; im_baggage_1_a: TImage;    im_baggage_2_a: TImage;    im_baggage_3_a: TImage;    im_baggage_4_a: TImage;    im_baggage_5_a: TImage; {60-65}
    im_baggage_0_b: TImage; im_baggage_1_b: TImage;    im_baggage_2_b: TImage;    im_baggage_3_b: TImage;    im_baggage_4_b: TImage;    im_baggage_5_b: TImage; {66-71}

                            im_miss_1_hero: TImage;    im_miss_2_hero: TImage;    im_miss_3_hero: TImage;    im_miss_4_hero: TImage;    im_miss_5_hero: TImage; {72-76}
                            im_miss_1_baggage: TImage; im_miss_2_baggage: TImage; im_miss_3_baggage: TImage; im_miss_4_baggage: TImage; im_miss_5_baggage: TImage; {77-81}

                            im_turtle_1_body: TImage;  im_turtle_2_body: TImage;  im_turtle_3_body: TImage;  im_turtle_4_body: TImage;  im_turtle_5_body: TImage; {82-86}
                            im_turtle_1_h_1: TImage;   im_turtle_2_h_1: TImage;   im_turtle_3_h_1: TImage;   im_turtle_4_h_1: TImage;   im_turtle_5_h_1: TImage; {87-91}
                            im_turtle_1_h_2: TImage;   im_turtle_2_h_2: TImage;   im_turtle_3_h_2: TImage;   im_turtle_4_h_2: TImage;   im_turtle_5_h_2: TImage; {92-96}
                            im_turtle_1_down: TImage;  im_turtle_2_down: TImage;  im_turtle_3_down: TImage;  im_turtle_4_down: TImage;  im_turtle_5_down: TImage; {97-101}

                            im_fish_1_1: TImage;       im_fish_2_1: TImage;       im_fish_3_1: TImage;       im_fish_4_1: TImage;       im_fish_5_1: TImage; {102-106}
                            im_fish_1_2: TImage;       im_fish_2_2: TImage;       im_fish_3_2: TImage;       im_fish_4_2: TImage;       im_fish_5_2: TImage; {107-111}
                            im_fish_1_3: TImage;       im_fish_2_3: TImage;       im_fish_3_3: TImage;       im_fish_4_3: TImage;       im_fish_5_3: TImage; {112-116}
                            im_fish_1_4: TImage;       im_fish_2_4: TImage;       im_fish_3_4: TImage;       im_fish_4_4: TImage;       im_fish_5_4: TImage; {117-121}
                            im_fish_1_5: TImage;       im_fish_2_5: TImage;       im_fish_3_5: TImage;       im_fish_4_5: TImage;       im_fish_5_5: TImage; {122-126}

{ * TIMERS                                                                   * }
    Timer_Bonus_Left:        TTimer;
    Timer_Bonus_Right:       TTimer;
    Timer_Chance_Time:       TTimer;
    Timer_Chance_Time_Blink: TTimer;
    Timer_Clock:             TTimer;
    Timer_Game:              TTimer;
    Timer_Game_Over:         TTimer;
    Timer_Hero_Down:         TTimer;
    Timer_Hero_Jump:         TTimer;
    Timer_Miss:              TTimer;
    Timer_Tourist_Right:     TTimer;
    Timer_Turtle_Head_1:     TTimer;
    Timer_Turtle_Head_2:     TTimer;
    Timer_Turtle_Head_3:     TTimer;
    Timer_Turtle_Head_4:     TTimer;
    Timer_Turtle_Head_5:     TTimer;

{ * POPUP MENU                                                               * }
    pm_0_menu:          TPopupMenu;
    pm_1_about:         TMenuItem;
    pm_2_readme:        TMenuItem;
    pm_3_whatsnew:      TMenuItem;
    pm_4_manual:        TMenuItem;
    pm_5_n1:            TMenuItem;
    pm_6_config:        TMenuItem;
    pm_7_keys:          TMenuItem;
    pm_8_hiscore:       TMenuItem;
    pm_9_n2:            TMenuItem;
  {}pm_10_fullscreen:   TMenuItem;
  {}pm_11_splashscreen: TMenuItem;
{}//pm_12_controller:   TMenuItem; (not required)
  {}pm_13_n3:           TMenuItem;
  {}pm_14_exit:         TMenuItem;

{ * PROCEDURES: INITIALIZE & SHUT DOWN                                       * }
    procedure FormCreate                    (Sender: TObject);
    procedure FormActivate                  (Sender: TObject);
    procedure FormClose                     (Sender: TObject; var Action: TCloseAction);

{ * PROCEDURES: DISPLAY MANAGEMENT                                           * }
    procedure Vid_Show_All                  (b: boolean);

    procedure Vid_Show_All_Hero             (b: boolean);
    procedure Vid_Show_All_Turtles          (b: boolean);
    procedure Vid_Show_All_Fish             (b: boolean);
    procedure Vid_Show_All_Miss             (b: boolean);

    procedure Vid_Show_Panel_Score          (var p: integer; b: boolean);
    procedure Vid_Show_Panel_Miss           (var p: integer);

    procedure Vid_Show_Hero_Down            (var p: integer; b1, b2: boolean);
    procedure Vid_Show_Hero_Jump            (var p: integer; b1, b2: boolean);

{ * PROCEDURES: AUDIO MANAGEMENT                                             * }
    procedure Snd_Play                      (pfs: PFSoundSample);
    procedure Snd_Silence;

{ * PROCEDURES: INPUT MANAGEMENT - EVENTS                                    * }
    procedure im_backgroundMouseDown        (Sender: TObject; Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
    procedure im_backgroundMouseMove        (Sender: TObject;                       Shift: TShiftState; X, Y: Integer);

    procedure btn_game_a_topMouseDown       (Sender: TObject; Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
    procedure btn_game_a_topMouseUp         (Sender: TObject; Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
    procedure btn_game_b_topMouseDown       (Sender: TObject; Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
    procedure btn_game_b_topMouseUp         (Sender: TObject; Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
    procedure btn_time_topMouseDown         (Sender: TObject; Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
    procedure btn_time_topMouseUp           (Sender: TObject; Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
    procedure btn_acl_topMouseDown          (Sender: TObject; Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
    procedure btn_acl_topMouseUp            (Sender: TObject; Button: TMouseButton; Shift: TShiftState; X, Y: Integer);

    procedure btn_left_topMouseDown         (Sender: TObject; Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
    procedure btn_left_topMouseUp           (Sender: TObject; Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
    procedure btn_right_topMouseDown        (Sender: TObject; Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
    procedure btn_right_topMouseUp          (Sender: TObject; Button: TMouseButton; Shift: TShiftState; X, Y: Integer);

    procedure FormKeyDown                   (Sender: TObject; var Key: Word;        Shift: TShiftState);
    procedure FormKeyUp                     (Sender: TObject; var Key: Word;        Shift: TShiftState);

{ * PROCEDURES: INPUT MANAGEMENT - ACTIONS                                   * }
    procedure Inp_game_a_down;              procedure Inp_game_a_up;
    procedure Inp_game_b_down;              procedure Inp_game_b_up;
    procedure Inp_time_down;                procedure Inp_time_up;
    procedure Inp_acl_down;                 procedure Inp_acl_up;

    procedure Inp_left_down;                procedure Inp_left_up;
    procedure Inp_right_down;               procedure Inp_right_up;

{ * PROCEDURES: GAME                                                         * }
    procedure Gam_Stop_All_Timers;

    procedure Gam_Check_Chance_Time;
    procedure Gam_Set_Difficulty;
    procedure Gam_Set_Game_Stage;
    procedure Gam_Set_Mode;

{ * PROCEDURES: TIMERS                                                       * }
    procedure Timer_Bonus_LeftTimer         (Sender: TObject);
    procedure Timer_Bonus_RightTimer        (Sender: TObject);
    procedure Timer_Chance_Time_BlinkTimer  (Sender: TObject);
    procedure Timer_Chance_TimeTimer        (Sender: TObject);
    procedure Timer_ClockTimer              (Sender: TObject);
    procedure Timer_GameTimer               (Sender: TObject);
    procedure Timer_Game_OverTimer          (Sender: TObject);
    procedure Timer_Hero_DownTimer          (Sender: TObject);
    procedure Timer_Hero_JumpTimer          (Sender: TObject);
    procedure Timer_MissTimer               (Sender: TObject);
    procedure Timer_Tourist_RightTimer      (Sender: TObject);
    procedure Timer_Turtle_Head_1Timer      (Sender: TObject);
    procedure Timer_Turtle_Head_2Timer      (Sender: TObject);
    procedure Timer_Turtle_Head_3Timer      (Sender: TObject);
    procedure Timer_Turtle_Head_4Timer      (Sender: TObject);
    procedure Timer_Turtle_Head_5Timer      (Sender: TObject);

{ * PROCEDURES: POPUP MENU MANAGEMENT                                        * }
    procedure pm_0_menuPopup                (Sender: TObject);
    procedure pm_1_aboutClick               (Sender: TObject);
    procedure pm_2_readmeClick              (Sender: TObject);
    procedure pm_3_whatsnewClick            (Sender: TObject);
    procedure pm_4_manualClick              (Sender: TObject);
    procedure pm_6_configClick              (Sender: TObject);
    procedure pm_7_keysClick                (Sender: TObject);
    procedure pm_8_hiscoreClick             (Sender: TObject);
  {}procedure pm_10_fullscreenClick         (Sender: TObject);
  {}procedure pm_11_splashscreenClick       (Sender: TObject);
{}//procedure pm_12_controllerClick         (Sender: TObject);
  {}procedure pm_14_exitClick               (Sender: TObject);

{ * PROCEDURES: GAME MENU MANAGEMENT                                         * }
    procedure btn_game_menu_topClick        (Sender: TObject);

    procedure la_menu_itemMouseEnter        (Sender: TObject);
    procedure la_menu_itemMouseLeave        (Sender: TObject);
    procedure la_menu_itemMouseUp           (Sender: TObject; Button: TMouseButton; Shift: TShiftState; X, Y: Integer);

    procedure Vid_Display_Game_Menu         (b: boolean);
  end;

const
{ introduced for use with HH2 / now incorporated into main source }

  HH2_CONTENT_NAME    = 'Turtle Bridge (Nintendo, Game & Watch Wide Screen)';
  HH2_CONTENT_AUTHOR  = 'Luca "MADrigal" Antignano';
  HH2_CONTENT_EMAIL   = 'lucantignano@gmail.com';
  HH2_CONTENT_VERSION = '5.00';
  HH2_CONTENT_DATE    = '2017/12/04';
  HH2_CONTENT_WEBSITE = 'www.madrigaldesign.it/sim';

  { constants for all builds }
  
  APP_NAME    = HH2_CONTENT_NAME + ' Simulator';                                // application's name (appears on taskbar and Splash screen)
{}APP_VERSION = 'Version ' + HH2_CONTENT_VERSION + ' * Final *';                // current version (appears on Splash screen)
{}APP_DATE    = 'Released by ' + HH2_CONTENT_AUTHOR + ' on 4th December, 2017'; // release date (appears on Splash screen)

  URL_EMAIL   = 'mailto:' + HH2_CONTENT_EMAIL;                                  // link to the email address
  URL_PAYPAL  = 'https://www.paypal.me/LucaAntignano';                          // link to the Paypal donation page
  URL_WEBSITE = 'http://' + HH2_CONTENT_WEBSITE + '/';                          // link to the website

  REG_KEY     = 'Software\MADrigal\TurtleBr50';                                 // registry key

  HINT_CLICK  = 'click or press';
  HINT_GAME_A = 'to start game A (hold to view highscore)';
  HINT_GAME_B = 'to start game B (hold to view highscore)';
  HINT_TIME   = 'to view current time (hold to view alarm time)';
  HINT_ACL    = 'to reset the game';
  HINT_ALARM  = 'button disabled';

{}MENU_ITEMS = 13;

var
{ * main objects * }
  Form1:                   TForm1;                                              // main form
  bStartup:                boolean;                                             // manages splash screen at startup
  iMouseX, iMouseY:        integer;                                             // cursor coordinates (to drag main form)
  iReg:                    TRegIniFile;                                         // system registry (to save high-scores)

{ * sound management * }
  pfs_chance,
  pfs_fish,
  pfs_gameover,
  pfs_miss,
  pfs_point,
  pfs_step:                PFSoundSample;                                       // sound samples

  bSound:                  boolean;                                             // sound enabled? (depending on selected driver on splash screen)

{ * arrays of images * }
  aIm_panel_alarm:         array[1..5]       of TImage;                         // alarm panel
  aIm_panel_miss:          array[0..3]       of TImage;                         // miss marks panel
  aIm_panel_digit_pos:     array[1..4]       of TImage;                         // empty sprites to host score/time digits
  aIm_panel_number:        array[0..9]       of TImage;                         // digits for score/time

  aIm_hero:                array[0..5]       of record
                                                  bodyup,
                                                  bodydown,
                                                  bagup,
                                                  bagdown: TImage;
                                                end;
  aIm_turtle:              array[1..5]       of record
                                                  bodyup,
                                                  bodydown,
                                                  headup,
                                                  headdown: TImage;
                                                end;
  aIm_fish:                array[1..5, 1..5] of TImage;
  aIm_miss:                array[0..5]       of record
                                                  hero,
                                                  baggage: TImage;
                                                end;

{ * arrays of timers * }
  aTimer_Turtle_Head:      array[1..5]       of TTimer;

{ * arrays of mixed types * }
  aKeys:                   array[1..6] of record                                // keys management (values)
                                            int:  integer;
                                            word: word;
                                           end;

  aKeyHints:               array[1..44] of string;                              // keys management (hint on buttons)

  aMenuItem:               array[1..MENU_ITEMS] of TMenuItem;                   // popup menu items
  aMenuLabels:             array[1..MENU_ITEMS] of TLabel;                      // on-screen game menu labels

{ * integers and booleans * }
  iMode,                                                                        // 0/1=ACL 1/2, 2=alarm, 3=time, 4/6=hiscore A/B, 5/7=game A/B, 8=game over
  iScore,                                                                       // score=0-9999
  iHiScoreA,                                                                    // hiscore for Game A=0-999
  iHiScoreB,                                                                    // hiscore for Game B=0-999
  iScoreBonusValue,                                                             // bonus points
  iScoreNextChance,                                                             // 200, 500, 1200, 1500, 2200 ... 9500

  iHeroPos,                                                                     // X coordinate of hero
  iMiss,                                                                        // number of misses: 0-3
  iMaxRandom                                                                    // maximum amount of fish on screen (depending on value of iScore)
                         : integer;

{}bFullscreenMode,                                                              // true if video mode is fullscreen (false if "shaped game")
  bGameMenuOpen,                                                                // true if the on-screen menu is visible

  bKeypressed,                                                                  // true while any left/right key is pressed
  bControls,                                                                    // true while player can actually move the hero

  bChance,                                                                      // true while the game is in "Chance-time"
  bHasBaggage                                                                   // true while the hero keeps the baggage on his shoulders
                         : boolean;

  // Game variables: loops
  iLoopBonusLeft,
  iLoopBonusRight,
  iLoopChanceTime,
  iLoopChanceTimeBlink,
  iLoopGame,
  iLoopGameBonusValue,
  iLoopGameOver,
  iLoopMiss,
  iLoopTouristRight
                         : integer;

implementation

uses
  about, gfxinit, keys, manual, splash;

{$r *.dfm}










{ **************************************************************************** }
{ *                                                                          * }
{ * PROCEDURES: INITIALIZE & SHUT DOWN                                       * }
{ *                                                                          * }
{ **************************************************************************** }

procedure TForm1.FormCreate(Sender: TObject);
{ sprites on form are associated to arrays of images, needed to "index" them   }
{ variables are initialized: system registry, randomize, game mode, etc.       }
  var
    i: integer;
begin
  // initializes application name (for Windows taskbar)
  Application.Title := APP_NAME;

{}// sets game off the screen while graphics initialise
{}Top  := - Screen.Height;
{}Left := - Screen.Width;

  // on-screen panel (alarm, time/score, game a/b, etc.)
  aIm_panel_alarm[1] := im_panel_alarm_1; aIm_panel_alarm[2] := im_panel_alarm_2; aIm_panel_alarm[3] := im_panel_alarm_3; aIm_panel_alarm[4] := im_panel_alarm_4; aIm_panel_alarm[5] := im_panel_alarm_5;

  aIm_panel_miss[0] := im_panel_miss_0; aIm_panel_miss[1] := im_panel_miss_1; aIm_panel_miss[2] := im_panel_miss_2; aIm_panel_miss[3] := im_panel_miss_3;

  aIm_panel_digit_pos[1] := im_panel_digit_1; aIm_panel_digit_pos[2] := im_panel_digit_2; aIm_panel_digit_pos[3] := im_panel_digit_3; aIm_panel_digit_pos[4] := im_panel_digit_4;

  aIm_panel_number[0] := im_panel_number_0; aIm_panel_number[1] := im_panel_number_1; aIm_panel_number[2] := im_panel_number_2; aIm_panel_number[3] := im_panel_number_3; aIm_panel_number[4] := im_panel_number_4; aIm_panel_number[5] := im_panel_number_5; aIm_panel_number[6] := im_panel_number_6; aIm_panel_number[7] := im_panel_number_7; aIm_panel_number[8] := im_panel_number_8; aIm_panel_number[9] := im_panel_number_9;

  // on-screen items
  with aIm_hero[0] do begin bodyup := im_hero_0_b; bodydown := im_hero_0_a; bagup := im_baggage_0_b; bagdown := im_baggage_0_a; end;
  with aIm_hero[1] do begin bodyup := im_hero_1_b; bodydown := im_hero_1_a; bagup := im_baggage_1_b; bagdown := im_baggage_1_a; end;
  with aIm_hero[2] do begin bodyup := im_hero_2_b; bodydown := im_hero_2_a; bagup := im_baggage_2_b; bagdown := im_baggage_2_a; end;
  with aIm_hero[3] do begin bodyup := im_hero_3_b; bodydown := im_hero_3_a; bagup := im_baggage_3_b; bagdown := im_baggage_3_a; end;
  with aIm_hero[4] do begin bodyup := im_hero_4_b; bodydown := im_hero_4_a; bagup := im_baggage_4_b; bagdown := im_baggage_4_a; end;
  with aIm_hero[5] do begin bodyup := im_hero_5_b; bodydown := im_hero_5_a; bagup := im_baggage_5_b; bagdown := im_baggage_5_a; end;

  aIm_miss[1].hero :=    im_miss_1_hero;    aIm_miss[2].hero :=    im_miss_2_hero;    aIm_miss[3].hero :=    im_miss_3_hero;    aIm_miss[4].hero :=    im_miss_4_hero;    aIm_miss[5].hero :=    im_miss_5_hero;
  aIm_miss[1].baggage := im_miss_1_baggage; aIm_miss[2].baggage := im_miss_2_baggage; aIm_miss[3].baggage := im_miss_3_baggage; aIm_miss[4].baggage := im_miss_4_baggage; aIm_miss[5].baggage := im_miss_5_baggage;

  with aIm_turtle[1] do begin bodyup := im_turtle_1_body; bodydown := im_turtle_1_down; headup := im_turtle_1_h_1; headdown := im_turtle_1_h_2; end;
  with aIm_turtle[2] do begin bodyup := im_turtle_2_body; bodydown := im_turtle_2_down; headup := im_turtle_2_h_1; headdown := im_turtle_2_h_2; end;
  with aIm_turtle[3] do begin bodyup := im_turtle_3_body; bodydown := im_turtle_3_down; headup := im_turtle_3_h_1; headdown := im_turtle_3_h_2; end;
  with aIm_turtle[4] do begin bodyup := im_turtle_4_body; bodydown := im_turtle_4_down; headup := im_turtle_4_h_1; headdown := im_turtle_4_h_2; end;
  with aIm_turtle[5] do begin bodyup := im_turtle_5_body; bodydown := im_turtle_5_down; headup := im_turtle_5_h_1; headdown := im_turtle_5_h_2; end;

  aIm_fish[1, 1] := im_fish_1_1; aIm_fish[2, 1] := im_fish_2_1; aIm_fish[3, 1] := im_fish_3_1; aIm_fish[4, 1] := im_fish_4_1; aIm_fish[5, 1] := im_fish_5_1;
  aIm_fish[1, 2] := im_fish_1_2; aIm_fish[2, 2] := im_fish_2_2; aIm_fish[3, 2] := im_fish_3_2; aIm_fish[4, 2] := im_fish_4_2; aIm_fish[5, 2] := im_fish_5_2;
  aIm_fish[1, 3] := im_fish_1_3; aIm_fish[2, 3] := im_fish_2_3; aIm_fish[3, 3] := im_fish_3_3; aIm_fish[4, 3] := im_fish_4_3; aIm_fish[5, 3] := im_fish_5_3;
  aIm_fish[1, 4] := im_fish_1_4; aIm_fish[2, 4] := im_fish_2_4; aIm_fish[3, 4] := im_fish_3_4; aIm_fish[4, 4] := im_fish_4_4; aIm_fish[5, 4] := im_fish_5_4;
  aIm_fish[1, 5] := im_fish_1_5; aIm_fish[2, 5] := im_fish_2_5; aIm_fish[3, 5] := im_fish_3_5; aIm_fish[4, 5] := im_fish_4_5; aIm_fish[5, 5] := im_fish_5_5;

  // timers
  aTimer_Turtle_Head[1] := Timer_Turtle_Head_1; aTimer_Turtle_Head[2] := Timer_Turtle_Head_2; aTimer_Turtle_Head[3] := Timer_Turtle_Head_3; aTimer_Turtle_Head[4] := Timer_Turtle_Head_4; aTimer_Turtle_Head[5] := Timer_Turtle_Head_5;

  // registry keys management
  iReg := TregIniFile.Create(REG_KEY);                                          // system registry init

{}// init graphics
{}bFullscreenMode := iReg.ReadBool('', 'FullscreenMode', true);
{}pm_10_fullscreen.Checked := bFullscreenMode;
{}Gfxinit.Gfx_Game_Init;
{}bGameMenuOpen := false;

{}// initialises the "hints" array
  for i := 1 to 10 do
    aKeyHints[i] := '''' + Chr(i + 47) + '''';

  for i := 11 to 36 do
    aKeyHints[i] := '''' + Chr(i + 54) + '''';

  aKeyHints[37] := '''up arrow'' key';
  aKeyHints[38] := '''down arrow'' key';
  aKeyHints[39] := '''left arrow'' key';
  aKeyHints[40] := '''right arrow'' key';
  aKeyHints[41] := '''alt'' key';
  aKeyHints[42] := '''ctrl'' key';
  aKeyHints[43] := '''shift'' key';
  aKeyHints[44] := '''spacebar''';

  // reads values from system registry into the inputs array
  aKeys[1].int := iReg.ReadInteger('', 'Key1', 2);                              // "Game A" (default: "1")
  aKeys[2].int := iReg.ReadInteger('', 'Key2', 3);                              // "Game B" (default: "2")
  aKeys[3].int := iReg.ReadInteger('', 'Key3', 4);                              // "Time"   (default: "3")
  aKeys[4].int := iReg.ReadInteger('', 'Key4', 6);                              // "ACL"    (default: "5")
  aKeys[5].int := iReg.ReadInteger('', 'Key5', 39);                             // "Left"   (default: "LEFT")
  aKeys[6].int := iReg.ReadInteger('', 'Key6', 40);                             // "Right"  (default: "RIGHT")

  // writes values into the system registry (in case no pre-existing values
  // stored), then assigns key values to the key-inputs array
  for i := 1 to 6 do
    begin
      iReg.WriteInteger('', 'Key' + inttostr(i), aKeys[i].int);
      KeysForm.AssignWord(i, aKeys[i].int);
    end;

  // sets button images hints
  btn_game_a_top.Hint := HINT_CLICK + ' ' + aKeyHints[aKeys[1].int] + ' ' + HINT_GAME_A; // "Game A"
  btn_game_b_top.Hint := HINT_CLICK + ' ' + aKeyHints[aKeys[2].int] + ' ' + HINT_GAME_B; // "Game B"
  btn_time_top.Hint   := HINT_CLICK + ' ' + aKeyHints[aKeys[3].int] + ' ' + HINT_TIME;   // "Time"
  btn_acl_top.Hint    := HINT_CLICK + ' ' + aKeyHints[aKeys[4].int] + ' ' + HINT_ACL;    // "ACL"
  btn_alarm_top.Hint  := HINT_ALARM;                                                     // "Alarm"
  btn_left_top.Hint   := HINT_CLICK + ' ' + aKeyHints[aKeys[5].int];                     // "Left"
  btn_right_top.Hint  := HINT_CLICK + ' ' + aKeyHints[aKeys[6].int];                     // "Right"

  // popup menu array association
  aMenuItem[1]  := pm_1_about;
  aMenuItem[2]  := pm_2_readme;
  aMenuItem[3]  := pm_3_whatsnew;
  aMenuItem[4]  := pm_4_manual;
  aMenuItem[5]  := pm_5_n1;
  aMenuItem[6]  := pm_6_config;
  aMenuItem[7]  := pm_7_keys;
  aMenuItem[8]  := pm_8_hiscore;
  aMenuItem[9]  := pm_9_n2;
{}aMenuItem[10] := pm_10_fullscreen;
{}aMenuItem[11] := pm_11_splashscreen;
{}//               pm_12_controller;
{}aMenuItem[12] := pm_13_n3;
{}aMenuItem[13] := pm_14_exit;

  // variables init
  DoubleBuffered := true;                                                       // enables double buffering for rendering of "form1"

  iHiScoreA := iReg.ReadInteger('', 'HiScore1', iHiScoreA);                     // reads value from system registry into iHiScoreA
  iHiScoreB := iReg.ReadInteger('', 'HiScore2', iHiScoreB);                     // reads value from system registry into iHiScoreB

  bControls := false;                                                           // prevents hero movement
  bKeyPressed := false;                                                         // no key is currently being kept pressed

  Randomize;                                                                    // initializes the random number generator
end;





procedure TForm1.FormActivate(Sender: TObject);
{ sends focus to the splash screen at startup                                  }
begin
  if bStartup then
    begin
      bStartup := false;
      SplashForm.Show;
    end;
end;





procedure TForm1.FormClose(Sender: TObject; var Action: TCloseAction);
{ frees memory before shutting down                                            }
begin
  FSOUND_Sample_Free(pfs_chance);   FSOUND_Sample_Free(pfs_fish);
  FSOUND_Sample_Free(pfs_gameover); FSOUND_Sample_Free(pfs_miss);
  FSOUND_Sample_Free(pfs_point);    FSOUND_Sample_Free(pfs_step);

  FSOUND_Close;

{}iReg.Destroy;
  Action := caFree;
end;










{ **************************************************************************** }
{ *                                                                          * }
{ * PROCEDURES: DISPLAY MANAGEMENT                                           * }
{ *                                                                          * }
{ **************************************************************************** }

procedure TForm1.Vid_Show_All(b: boolean);
{ displays/hides all sprites                                                   }
  var
    i: integer;
begin
  for i := 1 to 4 do
    aIm_panel_digit_pos[i].Visible := true;

  if b then
    begin
      aIm_panel_digit_pos[1].Picture := aIm_panel_number[1].Picture;
      aIm_panel_digit_pos[2].Picture := aIm_panel_number[2].Picture;
      aIm_panel_digit_pos[3].Picture := aIm_panel_number[0].Picture;
      aIm_panel_digit_pos[4].Picture := aIm_panel_number[0].Picture;
    end
  else
    for i := 1 to 4 do
      aIm_panel_digit_pos[i].Picture := nil;

  i := Ord(b) * 3; Vid_Show_Panel_Miss(i);

  for i := 1 to 5 do
    aIm_panel_alarm[i].Visible := b;

  im_panel_time_am.Visible := b;
  im_panel_time_pm.Visible := false;
  im_panel_time_colon.Visible := b;

  im_panel_game_a.Visible := b;
  im_panel_game_b.Visible := b;

  im_tourist_left.Visible := b;
  im_tourist_right_1.Visible := b;
  im_tourist_right_2.Visible := b;
  im_tourist_right_3.Visible := b;

  Vid_Show_All_Hero(b);
  Vid_Show_All_Turtles(b);
  Vid_Show_All_Fish(b);
  Vid_Show_All_Miss(b);
end;





procedure TForm1.Vid_Show_All_Hero(b: boolean);
{ displays/hides all hero's sprites at once                                    }
  var
    i: integer;
begin
  for i := 0 to 5 do
    begin
      aIm_hero[i].bodyup.Visible := b;
      aIm_hero[i].bodydown.Visible := b;
      aIm_hero[i].bagup.Visible := b;
      aIm_hero[i].bagdown.Visible := b;
    end;
end;





procedure TForm1.Vid_Show_All_Turtles(b: boolean);
{ displays/hides all turtles at once                                           }
  var
    i: integer;
begin
  for i := 1 to 5 do
    begin
      aIm_turtle[i].bodyup.Visible := b;
      aIm_turtle[i].bodydown.Visible := b;
      aIm_turtle[i].headup.Visible := b;
      aIm_turtle[i].headdown.Visible := b;
    end;
end;





procedure TForm1.Vid_Show_All_Fish(b: boolean);
{ displays/hides all fish at once                                              }
  var
    i, j: integer;
begin
  for i := 1 to 5 do
    for j :=  1 to 5 do
      aIm_fish[i, j].Visible := b;
end;





procedure TForm1.Vid_Show_All_Miss(b: boolean);
{ displays/hides all the sprites making the "miss" intermission                }
  var
    i: integer;
begin
  for i := 1 to 5 do
    begin
      aIm_miss[i].hero.Visible := b;
      aIm_miss[i].baggage.Visible := b;
    end;
end;





procedure TForm1.Vid_Show_Panel_Score(var p: integer; b: boolean);
{ transforms score/high-score into sprites, and displays/hides them            }
  var
    i, t: integer;
begin
  t := p;

  while t >= 10000 do                                                           // displays "10000" as "...0"
    Dec(t, 10000);

  if b then                                                                     // display or hide score/high-score?
    begin                                                                         // display
      if t > 999 then aIm_panel_digit_pos[1].Picture := aIm_panel_number[t div 1000 mod 10].Picture else aIm_panel_digit_pos[1].Picture := nil; // thousands
      if t >  99 then aIm_panel_digit_pos[2].Picture := aIm_panel_number[t div  100 mod 10].Picture else aIm_panel_digit_pos[2].Picture := nil; // hundreds
      if t >   9 then aIm_panel_digit_pos[3].Picture := aIm_panel_number[t div   10 mod 10].Picture else aIm_panel_digit_pos[3].Picture := nil; // tens
                      aIm_panel_digit_pos[4].Picture := aIm_panel_number[t          mod 10].Picture;                                            // units
    end
  else                                                                            // hide
    for i := 1 to 4 do
      aIm_panel_digit_pos[i].Picture := nil;
end;





procedure TForm1.Vid_Show_Panel_Miss(var p: integer);
{ displays misses panel in various configurations                              }
  var
    i: integer;
begin
  aIm_panel_miss[0].Visible := p > 0;                                           // displays/hides item 0

  for i := 1 to 3 do
    aIm_panel_miss[i].Visible := (i <= p);                                      // displays items 1 to p, hides the remaining ones
end;





procedure TForm1.Vid_Show_Hero_Down(var p: integer; b1, b2: boolean);
{ displays hero on a turtle; p: coordinate b1 for body, b2 for baggage         }
begin
  aIm_hero[p].bodydown.Visible := b1;
  aIm_hero[p].bagdown.Visible := b2;
end;





procedure TForm1.Vid_Show_Hero_Jump(var p: integer; b1, b2: boolean);
{ displays hero on a turtle; p: coordinate b1 for body, b2 for baggage         }
begin
  aIm_hero[p].bodyup.Visible := b1;
  aIm_hero[p].bagup.Visible := b2;
end;










{ **************************************************************************** }
{ *                                                                          * }
{ * PROCEDURES: AUDIO MANAGEMENT                                             * }
{ *                                                                          * }
{ **************************************************************************** }

procedure TForm1.Snd_Play(pfs: PFSoundSample);
{ plays a certain sound sample                                                 }
begin
  if bSound then
    FSOUND_PlaySound(FSOUND_FREE, pfs);
end;





procedure TForm1.Snd_Silence;
{ stops any currently playing sound sample                                     }
begin
  if bSound then
    FSOUND_StopSound(FSOUND_ALL);
end;










{ **************************************************************************** }
{ *                                                                          * }
{ * PROCEDURES: INPUT MANAGEMENT - EVENTS                                    * }
{ *                                                                          * }
{ **************************************************************************** }

procedure TForm1.im_backgroundMouseDown(Sender: TObject; Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
{ dragging the main form: PRESS                                                }
{ Note: this routine works if the game is not in 'Fullscreen' mode only        }
begin
{}if (not bFullscreenMode)
    and (Button = mbLeft) then
    begin
      iMouseX := X;
      iMouseY := Y;
    end;
end;





procedure TForm1.im_backgroundMouseMove(Sender: TObject; Shift: TShiftState; X, Y: Integer);
{ dragging the main form: MOVE                                                 }
{ Note: this routine works if the game is not in 'Fullscreen' mode only        }
begin
{}if (not bFullscreenMode)
    and (ssLeft in Shift) then
    begin
      Top  := Top  + Y - iMouseY;
      Left := Left + X - iMouseX;
    end;
end;





procedure TForm1.btn_game_a_topMouseDown(Sender: TObject; Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
{ game A button: PRESS                                                         }
begin
  if Button = mbLeft then
    Inp_game_a_down;
end;





procedure TForm1.btn_game_a_topMouseUp(Sender: TObject; Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
{ game A button: RELEASE                                                       }
begin
  if Button = mbLeft then
    Inp_game_a_up;
end;





procedure TForm1.btn_game_b_topMouseDown(Sender: TObject; Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
{ game B button: PRESS                                                         }
begin
  if Button = mbLeft then
    Inp_game_b_down;
end;





procedure TForm1.btn_game_b_topMouseUp(Sender: TObject; Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
{ game B button: RELEASE                                                       }
begin
  if Button = mbLeft then
    Inp_game_b_up;
end;





procedure TForm1.btn_time_topMouseDown(Sender: TObject; Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
{ time button: PRESS                                                           }
begin
  if Button = mbLeft then
    Inp_time_down;
end;





procedure TForm1.btn_time_topMouseUp(Sender: TObject; Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
{ time button: RELEASE                                                         }
begin
  if Button = mbLeft then
    Inp_time_up;
end;





procedure TForm1.btn_acl_topMouseDown(Sender: TObject; Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
{ ACL button: PRESS                                                            }
begin
  if Button = mbLeft then
    Inp_acl_down;
end;





procedure TForm1.btn_acl_topMouseUp(Sender: TObject; Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
{ ACL button: RELEASE                                                          }
begin
  if Button = mbLeft then
    Inp_acl_up;
end;





procedure TForm1.btn_left_topMouseDown(Sender: TObject; Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
{ LEFT button: PRESS                                                           }
begin
  if Button = mbLeft then
    Inp_left_down;
end;





procedure TForm1.btn_left_topMouseUp(Sender: TObject; Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
{ LEFT button: RELEASE                                                         }
begin
  if Button = mbLeft then
    Inp_left_up;
end;





procedure TForm1.btn_right_topMouseDown(Sender: TObject; Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
{ RIGHT button: PRESS                                                          }
begin
  if Button = mbLeft then
    Inp_right_down;
end;





procedure TForm1.btn_right_topMouseUp(Sender: TObject; Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
{ RIGHT button: RELEASE                                                        }
begin
  if Button = mbLeft then
    Inp_right_up;
end;





procedure TForm1.FormKeyDown(Sender: TObject; var Key: Word; Shift: TShiftState);
{ keypress: PRESS                                                              }
begin
  // note: can't use the "case..of" construct on the aKeys[n].word
  if Key = aKeys[1].word then Inp_game_a_down else                              // Game A
  if Key = aKeys[2].word then Inp_game_b_down else                              // Game B
  if Key = aKeys[3].word then Inp_time_down   else                              // Time
  if Key = aKeys[4].word then Inp_acl_down    else                              // ACL
  if Key = aKeys[5].word then Inp_left_down   else                              // left
  if Key = aKeys[6].word then Inp_right_down;                                   // right
end;





procedure TForm1.FormKeyUp(Sender: TObject; var Key: Word; Shift: TShiftState);
{ keypress: RELEASE                                                            }
begin
  // note: can't use the "case..of" construct on the aKeys[n].word
  if Key = aKeys[1].word then Inp_game_a_up else                                // Game A
  if Key = aKeys[2].word then Inp_game_b_up else                                // Game B
  if Key = aKeys[3].word then Inp_time_up   else                                // Time
  if Key = aKeys[4].word then Inp_acl_up    else                                // ACL
  if Key = aKeys[5].word then Inp_left_up   else                                // left
  if Key = aKeys[6].word then Inp_right_up;                                     // right
end;











{ **************************************************************************** }
{ *                                                                          * }
{ * PROCEDURES: INPUT MANAGEMENT - ACTIONS                                   * }
{ *                                                                          * }
{ **************************************************************************** }

procedure TForm1.Inp_game_a_down;
{ game A button: PRESS                                                         }
begin
  btn_game_a_down.Visible := true;

  if (iMode = 1) or                                                             // ACL/2     or
     (iMode = 3) or                                                             // time      or
     (iMode = 8) then                                                           // game over ->
    begin
      iMode := 4; Gam_Set_Mode;                                                   //         -> hiscore A
    end;
end;





procedure TForm1.Inp_game_a_up;
{ game A button: RELEASE                                                       }
begin
  btn_game_a_down.Visible := false;

  if iMode = 4 then                                                             // hiscore A ->
    begin
      iMode := 5; Gam_Set_Mode;                                                   //         -> game A
    end;
end;





procedure TForm1.Inp_game_b_down;
{ game B button: PRESS                                                         }
begin
  btn_game_b_down.Visible := true;

  if (iMode = 1) or                                                             // ACL/2     or
     (iMode = 3) or                                                             // time      or
     (iMode = 8) then                                                           // game over ->
    begin
      iMode := 6; Gam_Set_Mode;                                                   //         -> hiscore B
    end;
end;





procedure TForm1.Inp_game_b_up;
{ game B button: RELEASE                                                       }
begin
  btn_game_b_down.Visible := false;

  if iMode = 6 then                                                             // hiscore B ->
    begin
      iMode := 7; Gam_Set_Mode;                                                   //         -> game B
    end;
end;





procedure TForm1.Inp_time_down;
{ time button: PRESS                                                           }
begin
  btn_time_down.Visible := true;

  if (iMode = 1) or                                                             // ACL/2     or
     (iMode = 3) or                                                             // time      or
     (iMode = 8) then                                                           // game over ->
    begin
      iMode := 2; Gam_Set_Mode;                                                   //         -> alarm
    end;
end;





procedure TForm1.Inp_time_up;
{ time button: RELEASE                                                         }
begin
  btn_time_down.Visible := false;

  if iMode = 2 then                                                             // alarm ->
    begin
      iMode := 3; Gam_Set_Mode;                                                   //     -> time
    end;
end;





procedure TForm1.Inp_acl_down;
{ ACL button: PRESS                                                            }
begin
  btn_acl_down.Visible := true;

  iMode := 0; Gam_Set_Mode;                                                     // any mode -> ACL/1
end;





procedure TForm1.Inp_acl_up;
{ ACL button: RELEASE                                                          }
begin
  btn_acl_down.Visible := false;

  iMode := 1; Gam_Set_Mode;                                                     // ACL/1  -> ACL/2
end;





procedure TForm1.Inp_left_down;
{ LEFT button: PRESS                                                           }
begin
  btn_left_down.Visible := true;

  if ( (iMode = 5) or (iMode = 7) ) and bControls and (not bKeyPressed) then
    begin
      bKeypressed := true;

      if iHeroPos > 1 then
        begin
          Vid_Show_All_Hero(false);
          Dec(iHeroPos);
          Vid_Show_Hero_Jump(iHeroPos, true, bHasBaggage);

          Snd_Play(pfs_step);

          bControls := false;

          Timer_Hero_Jump.Interval := 580;
          Timer_Hero_Jump.Enabled := true;
        end
      else
        // checks collision hero-tourist on the left (bonus intermission)
        if (iHeroPos = 1) and (not bHasBaggage) then
          begin
            Gam_Stop_All_Timers;

            Vid_Show_All_Hero(false);
            iHeroPos := 0;
            Vid_Show_Hero_Jump(iHeroPos, true, bHasBaggage);

            Snd_Play(pfs_step);

            bControls := false;

            iLoopBonusLeft := 1;
            Timer_Bonus_Left.Interval := 300;
            Timer_Bonus_Left.Enabled := true;

            if not bChance then
              Timer_Game.Enabled := true;
          end;
    end;
end;





procedure TForm1.Inp_left_up;
{ LEFT button: RELEASE                                                         }
begin
  btn_left_down.Visible := false;

  bKeypressed := false;
end;





procedure TForm1.Inp_right_down;
{ RIGHT button: PRESS                                                          }
begin
  btn_right_down.Visible := true;

  if ( (iMode = 5) or (iMode = 7) ) and bControls and (not bKeyPressed) then
    begin
      Snd_Play(pfs_step);

      bKeypressed := true;
      bControls := false;

      // disables "force jump down of the left rock" timer
      if im_tourist_left.Visible then
        begin
          im_tourist_left.Visible := false;

          Timer_Hero_Down.Enabled := false;
        end;

      Vid_Show_All_Hero(false);
      Vid_Show_Hero_Jump(iHeroPos, true, bHasBaggage);

      if iHeroPos < 5 then
        Inc(iHeroPos);

      // checks collision hero-tourist on the right (bonus intermission)
      if im_tourist_right_2.Visible and aIm_hero[5].bagup.Visible then
        begin
          Timer_Tourist_Right.Enabled := false;

          iLoopBonusRight := 1;
          Timer_Bonus_Right.Interval := 320;
          Timer_Bonus_Right.Enabled := true;
        end
      else
        begin
          Timer_Hero_Jump.Interval := 580;
          Timer_Hero_Jump.Enabled := true;
        end;
    end;
end;





procedure TForm1.Inp_right_up;
{ RIGHT button: RELEASE                                                        }
begin
  btn_right_down.Visible := false;

  bKeypressed := false;
end;










{ **************************************************************************** }
{ *                                                                          * }
{ * PROCEDURES: GAME                                                         * }
{ *                                                                          * }
{ **************************************************************************** }

procedure TForm1.Gam_Stop_All_Timers;
{ stops all currently enabled timers                                           }
  var
    i: integer;
begin
  Timer_Bonus_Left.Enabled        := false;
  Timer_Bonus_Right.Enabled       := false;
  Timer_Chance_Time.Enabled       := false;
  Timer_Chance_Time_Blink.Enabled := false;
  Timer_Clock.Enabled             := false;
  Timer_Game.Enabled              := false;
  Timer_Game_Over.Enabled         := false;
  Timer_Hero_Down.Enabled         := false;
  Timer_Hero_Jump.Enabled         := false;
  Timer_Miss.Enabled              := false;
  Timer_Tourist_Right.Enabled     := false;

  for i := 1 to 5 do
    aTimer_Turtle_Head[i].Enabled := false;
end;





procedure TForm1.Gam_Check_Chance_Time;
{ stops action, sets the score value for the next "chance time", and executes  }
{ the "chance time" intermission"                                              }
  var
    i: integer;
begin
  if (not bChance) and (iScore >= iScoreNextChance) then
    begin
      // stops action
      Gam_Stop_All_Timers;

      Snd_Silence;

      bControls := false;

      // loops iScoreNextChance to 200, 500, 1200, 1500, ... , 9200, 9500
      i := iScore;

      while i > 1000 do
        Dec(i, 1000);

      if i >= 500 then
        Inc(iScoreNextChance, 700)
      else
        if i >= 200 then
          Inc(iScoreNextChance, 300);

      iLoopChanceTime := 1;
      Timer_Chance_Time.Interval := 100;
      Timer_Chance_Time.Enabled := true;
    end;
end;





procedure TForm1.Gam_Set_Difficulty;
{ calculates difficulty increase, depending on current score                   }
  var
    i: integer;
begin
  i := iScore;

  case iMode of
    5: begin
         if i < 100 then
           begin
             if                i <  10  then iMaxRandom := 1;
             if (i >= 10) and (i <  20) then iMaxRandom := 2;
             if (i >= 20) and (i <  30) then iMaxRandom := 3;
             if (i >= 30) and (i <  70) then iMaxRandom := 4;
             if  i >= 70                then iMaxRandom := 5;
           end
         else
           begin
             while i >= 100 do
               Dec(i, 100); // 0-99

             if                i <  10  then iMaxRandom := 2;
             if (i >= 10) and (i <  20) then iMaxRandom := 3;
             if (i >= 20) and (i <  70) then iMaxRandom := 4;
             if  i >= 70                then iMaxRandom := 5;
           end;

         Timer_Game.Interval := 300;
       end;
    7: if iScore < 200 then
         begin
           if                i <  10  then begin iMaxRandom := 1; Timer_Game.Interval := 315; end;
           if (i >= 10) and (i <  20) then       iMaxRandom := 2;
           if (i >= 20) and (i <  30) then       iMaxRandom := 3;
           if (i >= 30) and (i <  70) then       iMaxRandom := 4;
           if (i >= 70) and (i < 120) then       iMaxRandom := 5;
           if  i >= 120               then begin iMaxRandom := 6; Timer_Game.Interval := 260; end;
         end
       else
         begin
           while i >= 200 do
             Dec(i, 200); // 0-199

           if                i <  10  then begin iMaxRandom := 2; Timer_Game.Interval := 315; end;
           if (i >= 10) and (i <  20) then       iMaxRandom := 3;
           if (i >= 20) and (i <  70) then       iMaxRandom := 4;
           if (i >= 70) and (i < 120) then       iMaxRandom := 5;
           if  i >= 120               then begin iMaxRandom := 6; Timer_Game.Interval := 260; end;
         end;
  end; // case..of
end;





procedure TForm1.Gam_Set_Game_Stage;
{ sets the stage ready for the game                                            }
  var
    i: integer;
begin
  for i := 1 to 5 do
    begin
      aIm_turtle[i].bodyup.Visible := true;
      aIm_turtle[i].bodydown.Visible := false;
      aIm_turtle[i].headup.Visible := false;
      aIm_turtle[i].headdown.Visible := true;
    end;

  iHeroPos := 0;
  bHasBaggage := true;
  Vid_Show_Hero_Down(iHeroPos, true, bHasBaggage);

  im_tourist_left.Visible := true;

  // tourist on the right starts blinking
  im_tourist_right_1.Visible := false;
  im_tourist_right_2.Visible := false;
  im_tourist_right_3.Visible := false;

  iLoopTouristRight := 4; // begins with a 2-seconds delay
  Timer_Tourist_Right.Interval := 400;
  Timer_Tourist_Right.Enabled := true;
end;





procedure TForm1.Gam_Set_Mode;
{ sets game/time/ACL mode, depending on the value of the iMode variable        }
  var
    i: integer;
begin
  case iMode of
    0: begin                                                                    // ACL 1 (ACL button press)
         Gam_Stop_All_Timers;
         Snd_Silence;

         bControls := false;
       end;
    1: Vid_Show_All(true);                                                      // ACL 2 (ACL button release)
    2: begin                                                                    // ALARM (Time button press)
         Gam_Stop_All_Timers;

         Vid_Show_All(false);

         aIm_panel_alarm[1].Visible := true;
         aIm_panel_alarm[3].Visible := true;

         im_panel_time_pm.Visible := true;
         im_panel_time_colon.Visible := true;

         aIm_panel_digit_pos[1].Picture := aIm_panel_number[1].Picture;
         aIm_panel_digit_pos[2].Picture := aIm_panel_number[2].Picture;
         aIm_panel_digit_pos[3].Picture := aIm_panel_number[0].Picture;
         aIm_panel_digit_pos[4].Picture := aIm_panel_number[0].Picture;

         bControls := false;
       end;
    3: begin                                                                    // TIME (Time button release)
         Vid_Show_All(false);

         im_panel_time_colon.Visible := true;

         Timer_Clock.Interval := 1;
         Timer_Clock.Enabled := true;

         bControls := false;
       end;
 4, 6: begin                                                                    // HISCORE A/B (Game A button press)
         Gam_Stop_All_Timers;

         Vid_Show_All(false);

         Gam_Set_Game_Stage;

         case iMode of
           4: begin
                im_panel_game_a.Visible := true; Vid_Show_Panel_Score(iHiScoreA, true);
              end;
           6: begin
                im_panel_game_b.Visible := true; Vid_Show_Panel_Score(iHiScoreB, true);
              end;
         end; // case..of
         
         for i := 1 to 5 do
           begin
             aIm_turtle[i].headdown.Visible := false;
             aIm_turtle[i].headup.Visible := true;

             aTimer_Turtle_Head[i].Interval := 500;
             aTimer_Turtle_Head[i].Enabled := true;
           end;

         bControls := false;
       end;
 5, 7: begin                                                                    // GAME A/B (Game A/B button release)
         iScore := 0; Vid_Show_Panel_Score(iScore, true);
         iScoreBonusValue := 10;
         iScoreNextChance := 200;
         iMiss := 0;

         bChance := false;
         iLoopChanceTime := 0;

         Gam_Set_Difficulty;

         // action starts here!
         iLoopGame := 4; // always start with fish on the 4th column
         iLoopGameBonusValue := 1;
         Timer_Game.Enabled := true;

         // hero jumps down of the left rock in 8 seconds
         Timer_Hero_Down.Interval := 8000;
         Timer_Hero_Down.Enabled := true;

         bControls := true;
         bKeypressed := false;
       end;
    8: begin                                                                    // GAME OVER
         // do nothing
       end;
    end; // case..of
end;










{ **************************************************************************** }
{ *                                                                          * }
{ * PROCEDURES: TIMERS                                                       * }
{ *                                                                          * }
{ **************************************************************************** }

procedure TForm1.Timer_Bonus_LeftTimer(Sender: TObject);
{ "bonus score" intermission -> the hero has completed the "way back" course   }
  var
    i: integer;
begin
  case iLoopBonusLeft of
    1: begin
         Vid_Show_All_Hero(false);
         Vid_Show_Hero_Down(iHeroPos, true, bHasBaggage);

         iLoopBonusLeft := 2;
         Timer_Bonus_Left.Interval := 55;
       end;
    2: if iScoreBonusValue > 0 then
         begin
           Inc(iScore); Vid_Show_Panel_Score(iScore, true);

           Snd_Play(pfs_point);

           Gam_Set_Difficulty;
           Gam_Check_Chance_Time;

           // loops as long as "iScoreBonusValue" is above zero
           Dec(iScoreBonusValue);
         end
       else
         begin
           bHasBaggage := true;
           Vid_Show_Hero_Down(iHeroPos, true, bHasBaggage);

           im_tourist_left.Visible := true;

           iLoopBonusLeft := 3;
           Timer_Bonus_Left.Interval := 300;
         end;
    3: begin
         for i := 1 to 5 do
           begin
             aTimer_Turtle_Head[i].Interval := 500;
             aTimer_Turtle_Head[i].Enabled := true;
           end;

         iScoreBonusValue := 10;
         iLoopGameBonusValue := 1;

         bControls := true;

         // right tourist continues blinking
         Timer_Tourist_Right.Interval := 400;
         Timer_Tourist_Right.Enabled := true;

         // the hero jumps down of the rock on the left in 8 seconds
         Timer_Hero_Down.Interval := 8000;
         Timer_Hero_Down.Enabled := true;

         // reprise chance-time" if still active
         if bChance then
           begin
             Timer_Chance_Time_Blink.Interval := 800;
             Timer_Chance_Time_Blink.Enabled := true;
           end;

         Timer_Bonus_Left.Enabled := false;
       end;
  end; // case..of
end;





procedure TForm1.Timer_Bonus_RightTimer(Sender: TObject);
{ "bonus score" intermission -> the hero has completed the "way there" course  }
  var
    i: integer;
begin
  case iLoopBonusRight of
    1: begin
         Inc(iScore); Vid_Show_Panel_Score(iScore, true);

         Snd_Silence; Snd_Play(pfs_point);

         Gam_Set_Difficulty;
         Gam_Check_Chance_Time;

         iLoopBonusRight := 2;
       end;
    2: begin
         Inc(iScore); Vid_Show_Panel_Score(iScore, true);

         Snd_Play(pfs_point);

         Gam_Set_Difficulty;
         Gam_Check_Chance_Time;

         im_tourist_right_2.Visible := false; aIm_hero[5].bagup.Visible := false;
         im_tourist_right_3.Visible := true;
         iLoopBonusRight := 3;
       end;
    3: begin
         Inc(iScore); Vid_Show_Panel_Score(iScore, true);

         Snd_Play(pfs_point);

         Gam_Set_Difficulty;
         Gam_Check_Chance_Time;

         im_tourist_right_3.Visible := false;
         im_tourist_right_2.Visible := true;

         bHasBaggage := false;
         Vid_Show_All_Hero(false);
         Vid_Show_Hero_Down(iHeroPos, true, bHasBaggage);

         // if the rightmost turtle has diven down in the meantime...
         if aIm_turtle[iHeroPos].bodydown.Visible and aIm_hero[iHeroPos].bodydown.Visible then
           // a miss occurs
           begin
             Gam_Stop_All_Timers;
             bChance := false;

             iLoopMiss := 1;
             Timer_Miss.Interval := 100;
             Timer_Miss.Enabled := true;
           end
         else
           // ...else the rightmost turtle raises his head, and the hero lands on her
           begin
             aIm_turtle[5].headdown.Visible := false;
             aIm_turtle[5].headup.Visible := true;

             aTimer_Turtle_Head[5].Enabled := false;

             // forces turtles timers enable
             for i := 1 to 5 do
               begin
                 aTimer_Turtle_Head[i].Interval := 500;
                 aTimer_Turtle_Head[i].Enabled := true;
               end;

             // game continues if "chance-time" is not active
             if iLoopChanceTime = 0 then
               bControls := true;

             iLoopBonusRight := 0; // needed by the chance-time routines

             if bChance then
               begin
                 Timer_Chance_Time_Blink.Interval := 800;
                 Timer_Chance_Time_Blink.Enabled := true;
               end;

             Timer_Bonus_Right.Enabled := false;
           end;

         // right tourist continues blinking
         Timer_Tourist_Right.Interval := 400;
         Timer_Tourist_Right.Enabled := true;
       end;
  end; // case..of
end;





procedure TForm1.Timer_Chance_Time_BlinkTimer(Sender: TObject);
{ "chance time" duration. Score doesn't actually blink - not sure if that's    }
{ how the real game works...                                                   }
begin
  Inc(iLoopChanceTimeBlink);

  // if "20 + random amount" seconds have passed...
  if iLoopChanceTimeBlink = 25 then
    begin
      bChance := false;

      Timer_Game.Enabled := true;

      Timer_Chance_Time_Blink.Enabled := false;
    end;
end;





procedure TForm1.Timer_Chance_TimeTimer(Sender: TObject);
{ "chance time" intermission                                                   }
  var
    i: integer;
begin
  case iLoopChanceTime of
    1: begin
         Snd_Silence; Snd_Play(pfs_chance);

         Timer_Chance_Time.Interval := 250;

         // loops action
         iLoopChanceTime := 2;
       end;
 2..9: begin
         if iMiss = 1 then
           aIm_panel_miss[1].Visible := not aIm_panel_miss[1].Visible;

         if iMiss = 2 then
           begin
             aIm_panel_miss[1].Visible := not aIm_panel_miss[1].Visible;
             aIm_panel_miss[2].Visible := not aIm_panel_miss[2].Visible;
           end;

         // loops action
         Inc(iLoopChanceTime);
       end;
   10: begin
         if iMiss = 0 then
           // chance-time activated
           begin
             // brings all turtles up
             for i := 1 to 5 do
               begin
                 aIm_turtle[i].bodyup.Visible := true;
                 aIm_turtle[i].bodydown.Visible := false;
                 aIm_turtle[i].headup.Visible := false;
                 aIm_turtle[i].headdown.Visible := true;
               end;

             // chance-time lasts 20 seconds + random 20 (25 to 50 ticks of 800 msec. each)
             bChance := true;

             iLoopChanceTimeBlink := - Random(25);
             Timer_Chance_Time_Blink.Interval := 800;
             Timer_Chance_Time_Blink.Enabled := true;
           end
         else
           // resets misses and game continues
           begin
             iMiss := 0;

             Timer_Game.Enabled := true;
           end;

         Vid_Show_Panel_Miss(iMiss);

         iLoopChanceTime := 0;

         if iLoopBonusLeft = 2 then // bonus routine is still active
           Timer_Bonus_Left.Enabled := true;

         if iLoopBonusRight > 0 then
           begin
             Timer_Bonus_Right.Interval := 320;
             Timer_Bonus_Right.Enabled := true
           end
         else
           bControls := true;

         Timer_Chance_Time.Enabled := false;
       end;
  end // case..of
end;





procedure TForm1.Timer_ClockTimer(Sender: TObject);
{ transforms current time into "sprites" form, then displays it                }
  var
    present:              TDateTime;
    hour, min, sec, msec: Word;
begin
  present := Now;                                                               // gets system time...
  DecodeTime(present, hour, min, sec, msec);                                    // and decodes it

  if hour > 11 then                                                             // changes mode from 0/24 to 12 AM/12 PM
    begin
      Dec(hour, 12);
      im_panel_time_am.Visible := false;
      im_panel_time_pm.Visible := true;
    end
  else
    begin
      im_panel_time_am.Visible := true;
      im_panel_time_pm.Visible := false;
    end;

  if hour = 0 then                                                              // changes mode from 0/24 to 12 AM/12 PM
    hour := 12;

  if hour > 9 then                                                              // hour: tens
    aIm_panel_digit_pos[1].Picture := aIm_panel_number[hour div 10].Picture
  else
    aIm_panel_digit_pos[1].Picture := nil;

  aIm_panel_digit_pos[2].Picture := aIm_panel_number[hour mod 10].Picture;      // hour: units
  aIm_panel_digit_pos[3].Picture := aIm_panel_number[min  div 10].Picture;      // minute: tens
  aIm_panel_digit_pos[4].Picture := aIm_panel_number[min  mod 10].Picture;      // minute: units

  Timer_Clock.Interval := 1000;
end;





procedure TForm1.Timer_GameTimer(Sender: TObject);
{ main game action                                                             }
  var
    i, j,
    iRnd,
    iFishCounter: integer;

    bFishMoved: boolean;
begin
  // inits audio routine
  bFishMoved := false;

  // fish counter
  iFishCounter := 0;

  for i := 1 to 5 do
    for j := 1 to 5 do
      if aIm_fish[i, j].Visible then
        Inc(iFishCounter);

  // turtles animation
  if aIm_turtle[iLoopGame].bodydown.Visible then // turtle has just eaten
    begin
      aIm_turtle[iLoopGame].bodyup.Visible := true;
      aIm_turtle[iLoopGame].bodydown.Visible := false;
      aIm_turtle[iLoopGame].headup.Visible := true;
      aIm_turtle[iLoopGame].headdown.Visible := false;

      aTimer_Turtle_Head[iLoopGame].Interval := 500;
      aTimer_Turtle_Head[iLoopGame].Enabled := true;
    end;

  if aIm_fish[iLoopGame, 1].Visible then // turtle eats fish now
    begin
      aIm_fish[iLoopGame, 1].Visible := false;

      aIm_turtle[iLoopGame].bodyup.Visible := false;
      aIm_turtle[iLoopGame].bodydown.Visible := true;
      aIm_turtle[iLoopGame].headup.Visible := false;
      aIm_turtle[iLoopGame].headdown.Visible := false;

      aTimer_Turtle_Head[iLoopGame].Enabled := false;  // unneeded (for more safety...)

      bFishMoved := true;
    end;

  // fish movement
  for i := 1 to 4 do
    begin
      aIm_fish[iLoopGame, i].Visible := aIm_fish[iLoopGame, i + 1].Visible;

      if aIm_fish[iLoopGame, i].Visible then
        bFishMoved := true;
    end;

  // fish generation
  iRnd := Random(3);                                                            // 33% chance
  if iFishCounter = 0 then iRnd := 0;                                           // no fish on screen                  -> FORCE YES
  if iFishCounter >= iMaxRandom then iRnd := 1;                                 // too many  fish on screen           -> FORCE NO
  if aIm_fish[iLoopGame, 4].Visible then iRnd := 1;                             // fish on same column, row 4         -> FORCE NO
  if (iLoopGame <= 4) and aIm_fish[iLoopGame + 1, 5].Visible then iRnd := 1;    // fish on column + 1, row 5 (part 1) -> FORCE NO
  if (iLoopGame = 5) and aIm_fish[4, 5].Visible then iRnd := 1;                 // fish on column + 1, row 5 (part 2) -> FORCE NO
  if (iLoopGame = 3) and (iMode = 5) then iRnd := 1;                            // fish on 3rd column, game A         -> FORCE NO

  if iRnd = 0 then
    begin
      aIm_fish[iLoopGame, 5].Visible := true;
      bFishMoved := true;
    end
  else
    aIm_fish[iLoopGame, 5].Visible := false;

  // play sound sample
  if bFishMoved then
    Snd_Play(pfs_fish);

  // error check
  if aIm_turtle[iHeroPos].bodydown.Visible and aIm_hero[iHeroPos].bodydown.Visible then
    begin
      Gam_Stop_All_Timers;

      bControls := false;
      bChance := false;

      iLoopMiss := 1;
      Timer_Miss.Interval := 100;
      Timer_Miss.Enabled := true;

      // right tourist continues blinking
      Timer_Tourist_Right.Interval := 400;
      Timer_Tourist_Right.Enabled := true;
    end
  else
    begin
      // calculates bonus score
      Inc(iLoopGameBonusValue);

      case iLoopGameBonusValue of
        30: iScoreBonusValue := 8; // every fish moved 6 times
        40: iScoreBonusValue := 6; // every fish moved 8 times
        50: iScoreBonusValue := 4; // every fish moved 10 times
        60: iScoreBonusValue := 2; // every fish moved 12 times (and over...)
      end; // case..of

      // loops action
      Dec(iLoopGame);

      if iLoopGame = 0
        then iLoopGame := 5;
    end;
end;





procedure TForm1.Timer_Game_OverTimer(Sender: TObject);
{ last part of the "miss" intermission, then game goes into "time" mode, five  }
{ minutes after game is over                                                   }
begin
  case iLoopGameOver of
    1: begin
         Snd_Silence; // needed to stop playing the previous sound effect
         Snd_Play(pfs_gameover);

         Timer_Game_Over.Interval := 1500;
       end;
    2: begin
         // sets high scores
         if (iMode = 5) and (iScore > iHiScoreA) then
           begin
             iHiScoreA := iScore;
             iReg.WriteInteger('', 'HiScore1', iHiScoreA);
           end;

         if (iMode = 7) and (iScore > iHiScoreB) then
           begin
             iHiScoreB := iScore;
             iReg.WriteInteger('', 'HiScore2', iHiScoreB);
           end;

         // right tourist continues blinking
         Timer_Tourist_Right.Interval := 400;
         Timer_Tourist_Right.Enabled := true;

         // sets game mode to "game over"
         iMode := 8; Gam_Set_Mode;

         Timer_Game_Over.Interval := 300000;
       end;
    3: begin
         iMode := 3; Gam_Set_Mode;

         Timer_Game_Over.Enabled := false;
       end;
  end; // case..of

  // loops action
  Inc(iLoopGameOver);
end;





procedure TForm1.Timer_Hero_DownTimer(Sender: TObject);
{ hero is jumping                                                              }
begin
  Vid_Show_Hero_Down(iHeroPos, false, false);
  Vid_Show_Hero_Jump(iHeroPos, true, bHasBaggage);

  Snd_Play(pfs_step);

  bControls := false;

  iHeroPos := 1;
  Timer_Hero_Jump.Interval := 580;
  Timer_Hero_Jump.Enabled := true;

  Timer_Hero_Down.Enabled := false;
end;





procedure TForm1.Timer_Hero_JumpTimer(Sender: TObject);
{ hero is landing on the turtle                                                }
begin
  Vid_Show_All_Hero(false);
  Vid_Show_Hero_Down(iHeroPos, true, bHasBaggage);

  if aIm_turtle[iHeroPos].bodyup.Visible then
    begin
      aIm_turtle[iHeroPos].headdown.Visible := false;
      aIm_turtle[iHeroPos].headup.Visible := true;

      bControls := true;

      aTimer_Turtle_Head[iHeroPos].Interval := 500;
      aTimer_Turtle_Head[iHeroPos].Enabled := false;
      aTimer_Turtle_Head[iHeroPos].Enabled := true;
    end
  else
    // a miss has occurred!
    begin
      Gam_Stop_All_Timers;
      bChance := false;

      iLoopMiss := 1;
      Timer_Miss.Interval := 100;
      Timer_Miss.Enabled := true;
    end;

  Timer_Hero_Jump.Enabled := false;
end;





procedure TForm1.Timer_MissTimer(Sender: TObject);
{ a miss has occurred. executes "miss" intermission                            }
begin
  case iLoopMiss of
    1: begin
         Vid_Show_Hero_Down(iHeroPos, false, false);
         aIm_miss[iHeroPos].hero.Visible := true;
         aIm_miss[iHeroPos].baggage.Visible := bHasBaggage;
       end;
    2: Snd_Play(pfs_miss);
   14: begin
         Inc(iMiss); Vid_Show_Panel_Miss(iMiss);

         if iMiss = 3 then
           begin
             iLoopGameOver := 1;
             Timer_Game_Over.Interval := 80;
             Timer_Game_Over.Enabled := true;

             Timer_Tourist_Right.Enabled := false;

             Timer_Miss.Enabled := false;
           end;
       end;
   23: Vid_Show_All_Miss(false);
   26: begin
         bHasBaggage := true;

         iHeroPos := 0;
         Vid_Show_Hero_Down(iHeroPos, true, bHasBaggage);

         iScoreBonusValue := 10;
         iLoopGameBonusValue := 1;

         im_tourist_left.Visible := true;

         bControls := true;

         Timer_Game.Enabled := true;

         Timer_Hero_Down.Interval := 8000;
         Timer_Hero_Down.Enabled := true;

         // right tourist continues blinking
         Timer_Tourist_Right.Interval := 400;
         Timer_Tourist_Right.Enabled := true;

         Timer_Miss.Enabled := false;
       end;
  end; // case..of

  // loops action
  Inc(iLoopMiss);
end;





procedure TForm1.Timer_Tourist_RightTimer(Sender: TObject);
{ tourist on the rightmost side of the screen blinks                           }
begin
  // displays tourist at every 4 seconds
  if iLoopTouristRight = 11 then
    begin
      im_tourist_right_1.Visible := not im_tourist_right_1.Visible;
      im_tourist_right_2.Visible := not im_tourist_right_2.Visible;
      iLoopTouristRight := 0; // becomes 1 with next "inc..."
    end;

  // checks collision hero-tourist (bonus intermission)
  if im_tourist_right_2.Visible and aIm_hero[5].bagup.Visible then
    begin
      Timer_Hero_Jump.Enabled := false;
      Timer_Tourist_Right.Enabled := false;

      iLoopBonusRight := 1;
      Timer_Bonus_Right.Interval := 320;
      Timer_Bonus_Right.Enabled := true;
    end;

  // loops action
  Inc(iLoopTouristRight);
end;





{ turtles move their head down                                                 }
procedure TForm1.Timer_Turtle_Head_1Timer(Sender: TObject); begin if aIm_turtle[1].bodyup.Visible then begin aIm_turtle[1].headup.Visible := false; aIm_turtle[1].headdown.Visible := true; end; Timer_Turtle_Head_1.Enabled := false; end;
procedure TForm1.Timer_Turtle_Head_2Timer(Sender: TObject); begin if aIm_turtle[2].bodyup.Visible then begin aIm_turtle[2].headup.Visible := false; aIm_turtle[2].headdown.Visible := true; end; Timer_Turtle_Head_2.Enabled := false; end;
procedure TForm1.Timer_Turtle_Head_3Timer(Sender: TObject); begin if aIm_turtle[3].bodyup.Visible then begin aIm_turtle[3].headup.Visible := false; aIm_turtle[3].headdown.Visible := true; end; Timer_Turtle_Head_3.Enabled := false; end;
procedure TForm1.Timer_Turtle_Head_4Timer(Sender: TObject); begin if aIm_turtle[4].bodyup.Visible then begin aIm_turtle[4].headup.Visible := false; aIm_turtle[4].headdown.Visible := true; end; Timer_Turtle_Head_4.Enabled := false; end;
procedure TForm1.Timer_Turtle_Head_5Timer(Sender: TObject); begin if aIm_turtle[5].bodyup.Visible then begin aIm_turtle[5].headup.Visible := false; aIm_turtle[5].headdown.Visible := true; end; Timer_Turtle_Head_5.Enabled := false; end;










{ **************************************************************************** }
{ *                                                                          * }
{ * PROCEDURES: POPUP MENU MANAGEMENT                                        * }
{ *                                                                          * }
{ **************************************************************************** }

procedure TForm1.pm_0_menuPopup(Sender: TObject);
{ shows the "checked" menuitems if needed. also, hides the game menu if it is  }
{ visible when the popup menu pops up                                          }
begin
{}pm_11_splashscreen.Checked := iReg.ReadBool('', 'SplashScreen', SplashForm.chb_splashscreen.Checked);

{}if bFullscreenMode
    and bGameMenuOpen then
    begin
      bGameMenuOpen := false;
      Form1.Vid_Display_Game_Menu(bGameMenuOpen);
    end;
end;





procedure TForm1.pm_1_aboutClick(Sender: TObject);
{ displays the embedded "About the author..." form                             }
begin
  AboutForm.Show;
end;





procedure TForm1.pm_2_readmeClick(Sender: TObject);
{ opens the "Readme.txt" file                                                  }
begin
  ShellExecute(Handle, 'open', 'Readme.txt', nil, '', SW_SHOWNORMAL);
end;





procedure TForm1.pm_3_whatsnewClick(Sender: TObject);
{ opens the "Whatsnew.txt" file                                                }
begin
  ShellExecute(Handle, 'open', 'Whatsnew.txt', nil, '', SW_SHOWNORMAL);
end;





procedure TForm1.pm_4_manualClick(Sender: TObject);
{ displays the embedded instructions manual                                    }
begin
  ManualForm.Show;
end;





procedure TForm1.pm_6_configClick(Sender: TObject);
{ opens the "Config.ini" file                                                  }
begin
  ShellExecute(Handle, 'open', 'Config.ini', nil, '', SW_SHOWNORMAL);
end;





procedure TForm1.pm_7_keysClick(Sender: TObject);
{ displays the "key settings" form                                             }
begin
  KeysForm.position := poScreenCenter;
  KeysForm.visible := true;
  Form1.Enabled := false;
end;





procedure TForm1.pm_8_hiscoreClick(Sender: TObject);
{ resets high scores to the system registry                                    }
begin
  iHiScoreA := 0; iReg.WriteInteger('', 'HiScore1', iHiScoreA);
  iHiScoreB := 0; iReg.WriteInteger('', 'HiScore2', iHiScoreB);
  Beep;
end;





{}procedure TForm1.pm_10_fullscreenClick(Sender: TObject);
{ changes from fullscreen to windowed, and vice-versa                          }
begin
  // updates popup menu
  bFullscreenMode := not bFullscreenMode;
  pm_10_fullscreen.Checked := bFullscreenMode;
  bGameMenuOpen := false;

  iReg.WriteBool('', 'FullscreenMode', bFullscreenMode);

  // graphics init
  Top  := - Screen.Height;
  Left := - Screen.Width;
  Gfx_Game_Init;
  Position := poScreenCenter;
end;





procedure TForm1.pm_11_splashscreenClick(Sender: TObject);
{ updates the ticks to the checkbox, the popup menu and the system registry    }
begin
{}pm_11_splashscreen.Checked := not pm_11_splashscreen.Checked;

{}iReg.WriteBool('', 'SplashScreen', pm_11_splashscreen.Checked);
end;





procedure TForm1.pm_14_exitClick(Sender: TObject);
{ exits the game                                                               }
begin
  Application.Terminate;
end;










{ **************************************************************************** }
{ *                                                                          * }
{ * PROCEDURES: GAME MENU MANAGEMENT                                         * }
{ *                                                                          * }
{ **************************************************************************** }

procedure TForm1.btn_game_menu_topClick(Sender: TObject);
{ sets the game menu as visible/hidden, then displays/hides it                 }
begin
  bGameMenuOpen := not bGameMenuOpen;
  Vid_Display_Game_Menu(bGameMenuOpen);
end;





procedure TForm1.la_menu_itemMouseEnter(Sender: TObject);
{ the game menu label bg colour becomes solid                                  }
begin
  if Sender is TLabel then
    TLabel(Sender).Transparent := false;
end;





procedure TForm1.la_menu_itemMouseLeave(Sender: TObject);
{ the game menu label bg colour becomes transparent                            }
begin
  if Sender is TLabel then
    TLabel(Sender).Transparent := true;
end;





procedure TForm1.la_menu_itemMouseUp(Sender: TObject; Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
{ hides the game menu, once a label has been pressed                           }
begin
  if Button = mbLeft then
    begin
      bGameMenuOpen := false;
      Vid_Display_Game_Menu(false);
    end;
end;





procedure TForm1.Vid_Display_Game_Menu(b: boolean);
{ displays/hides the game menu, depending on the value of bGameMenuOpen        }
  var
    i: integer;
begin
  case bGameMenuOpen of
    true: begin
        for i := 1 to MENU_ITEMS do
          begin
            aMenuLabels[i] := TLabel.Create(Form1);
            aMenuLabels[i].Parent := Form1;

            aMenuLabels[i].Top := 32 * i;
            aMenuLabels[i].Left := 0;

            aMenuLabels[i].AutoSize := false;
            aMenuLabels[i].Height := 32;
            aMenuLabels[i].Width := 256;
            aMenuLabels[i].Layout := tlCenter;

            aMenuLabels[i].Transparent := true;

            if aMenuItem[i].Caption <> '-' then
              begin
                aMenuLabels[i].Font.Name := 'Arial';
                aMenuLabels[i].Font.Color := clWhite;
                aMenuLabels[i].Font.Height := 16;
                aMenuLabels[i].Caption := '   ' + aMenuItem[i].Caption;

                if aMenuItem[i].Checked then
                  aMenuLabels[i].Caption := aMenuLabels[i].Caption + ' [V]';

                aMenuLabels[i].Color := $00000000;
                aMenuLabels[i].Cursor := crHandPoint;
                aMenuLabels[i].Enabled := aMenuItem[i].Enabled;

                if aMenuLabels[i].Enabled then
                  begin
                    aMenuLabels[i].OnClick      := aMenuItem[i].OnClick;
                    aMenuLabels[i].OnMouseEnter := la_menu_itemMouseEnter;
                    aMenuLabels[i].OnMouseLeave := la_menu_itemMouseLeave;
                    aMenuLabels[i].OnMouseUp    := la_menu_itemMouseUp;
                  end;
              end;
          end;

        btn_game_menu_background.Width := 256;
        btn_game_menu_background.Height := 32 * (MENU_ITEMS + 1)
      end;
    false: begin
        for i := 1 to MENU_ITEMS do
          begin
            aMenuLabels[i].Free;
          end;

        btn_game_menu_background.Height := 32;
        btn_game_menu_background.Width := 36;
      end;
    end; // case..of
end;





initialization
  bStartup := true;





end.
